/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_RENDER_H__
#define __KIBA_RENDER_H__

/* rerender */
void
kiba_render (KibaDock *dock);

/* rerender the dock */
void
kiba_dock_paint (KibaDock *dock, cairo_t *cr);

/* clear dock window and object window */
void
kiba_clear_windows (KibaDock *dock);

/* update the dock surface */
void
kiba_update_dock_surface (KibaDock   *dock,
                          cairo_t    *cr);

/* draws svg_handle or GdkPixpuf on drawing_context */
void
kiba_draw_icon_surface_buffer (KibaDock   *dock,
                               KibaObject *object,
                               cairo_t    *drawing_context,
                               gboolean    background);

/* on background == TRUE the function fills object->bg_surface else it updates object->fg_surface */
void
kiba_update_surface_buffer (KibaDock   *dock,
                            KibaObject *object,
                            cairo_t    *pSourceContext,
                            gboolean    background);

/* update the prelight surface */
void
kiba_update_prelight_surface (KibaDock   *dock,
                              cairo_t    *cr);

/* rotate context depending on dock position */
void
kiba_rotate_context (KibaDock *dock,
                     cairo_t  *cr,
                     double    width,
                     double    height);

/* creates a cairo context on a offscreen pixmap, or the glitz surface */
cairo_t *
kiba_cairo_create (KibaDock *dock);

#ifdef HAVE_SVG
/* resize svg handle */
void
rsvg_resize_func (gint     *width,
                  gint     *height,
                  gpointer  user_data);
#endif

/* draws a small Triangle on the bottom of the given Object */
void
kiba_draw_triangle (KibaDock   *dock,
                    KibaObject *object,
                    cairo_t    *drawing_context);

/* stroke shape */
void
kiba_paint_shape (KibaDock   *dock,
                  cairo_t    *draw_context,
                  const char *topleft_corner,        // *_corner can be "rounded" "squared" "notched" "mitered"
                  const char *topright_corner,
                  const char *bottomleft_corner,
                  const char *bottomright_corner,
                  gboolean    dock_shape,
                  gboolean    draw_all_border,
                  double      x0,
                  double      y0,
                  double      x1,
                  double      y1,
                  double      radius,
                  double      border_width);

cairo_pattern_t*
kiba_get_gradient_pattern (KibaDock   *dock,
                           const char *dock_color_1,
                           const char *dock_color_2,
                           const char *dock_color_3,
                           double      dock_alpha_1,
                           double      dock_alpha_2,
                           double      dock_alpha_3,
                           double      x0,
                           double      x1,
                           double      y0,
                           double      y1);

/* creates cairo_pattern_t* for the given png file png_file */
cairo_pattern_t*
kiba_get_png_pattern (KibaDock   *dock,
                      cairo_t    *cr,
                      const char *png_file,
                      gboolean    rotate);

/* creates linear or radial pattern */
cairo_pattern_t*
kiba_get_bg_pattern (KibaDock   *dock,
                     gboolean    linear,
                     const char *color1,
                     const char *color2,
                     const char *color3,
                     const char *color4,
                     const char *color5,
                     double      alpha1,
                     double      alpha2,
                     double      alpha3,
                     double      alpha4,
                     double      alpha5,
                     double      affection,
                     double      step_size,
                     double      radius_1,
                     double      radius_2);

#endif
