/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_GCONF_H__
#define __KIBA_GCONF_H__

#ifndef __KIBA_DOCK_H__
  typedef struct _KibaSettings     KibaSettings;
#endif

typedef struct _GconfOption   GconfOption;

struct _GconfOption {
  const char *name;
  size_t      offset;
};

struct _KibaSettings {
  GList           *plugin_options;

  /* STYLE */
  gboolean         draw_dock;
  gboolean         animated_background;
  gboolean         show_all_borders;
  char            *dock_border_color;
  char            *dock_background;
  char            *dock_color_1;
  char            *dock_color_2;
  char            *dock_color_3;
  char            *dock_corner_topleft;
  char            *dock_corner_topright;
  char            *dock_corner_bottomleft;
  char            *dock_corner_bottomright;
  char            *dock_position;
  int              dock_cap_margin;
  int              dock_cap_size;
  double           dock_margin;
  int              dock_position_stepsize;
  int              monitor_number;
  double           border_alpha;
  double           border_width;
  double           dock_alpha_1;
  double           dock_alpha_2;
  double           dock_alpha_3;
  double           dock_bg_affection;
  double           dock_bg_step_size;
  double           dock_bg_radius_1;
  double           dock_bg_radius_2;
  double           dock_background_alpha;
  double           mouse_translation;
  double           object_space;
  double           object_margin;
  double           object_animate_stepsize;
  double           plugin_space;
  double           dock_edge_radius;

  /* Notification Bubble */
  char            *bubble_color;
  char            *bubble_text_color;
  char            *bubble_icon_edge;
  double           bubble_alpha;
  double           bubble_text_alpha;

  /* Notification Rectangle */
  char            *notification_rectangle_color;
  double           notification_rectangle_alpha;

  /* Notification Arrow */
  char            *arrow_background;
  double           arrow_bg_step_size;
  double           arrow_bg_affection;
  double           arrow_bg_radius_1;
  double           arrow_bg_radius_2;

  /* AUTOHIDE */
  gboolean         auto_hide_enable;
  int              auto_hide_area;
  int              auto_hide_delay;
  int              auto_hide_speed;

  /* PHYSICS */
  gboolean         use_allways_physics;
  gboolean         rope_spring_while_dragging;
  char            *object_model;
  double           rope_object_distance;
  double           throw_gravity;
  double           gravity_multiplier;
  double           inertia;
  int              wobbly_object_spring;
  int              rope_gravity;

  /* TEXT */
  gboolean         text_paint;
  char            *text_color;
  char            *text_shadow_color;
  char            *text_background;
  char            *text_border_style;
  char            *text_bg_color_1;
  char            *text_bg_color_2;
  char            *text_bg_color_3;
  char            *text_bg_color_4;
  char            *text_bg_color_5;
  char            *text_border_color;
  char            *text_placement_x;
  char            *text_placement_y;
  char            *text_font_type;
  char            *text_font_weight;
  int              text_shadow_offset;
  int              text_position_stepsize;
  int              text_font_size;
  int              text_icon_padding;
  int              text_hide_delay;
  double           text_alpha;
  double           text_shadow_alpha;
  double           text_bg_alpha_1;
  double           text_bg_alpha_2;
  double           text_bg_alpha_3;
  double           text_bg_alpha_4;
  double           text_bg_alpha_5;
  double           text_bg_radius_1;
  double           text_bg_radius_2;
  double           text_bg_step_size;
  double           text_bg_affection;
  double           text_border_alpha;
  double           text_border_width;

  char            *mouse_over_effect;

  /* PULSE */
  gboolean         pulse_exec_enable;
  double           pulse_exec_stepsize;
  double           pulse_exec_zoom_factor;
  double           pulse_stepsize;
  double           pulse_max_alpha;
  double           pulse_scale_factor;

  /* ZOOM */
  double           zoom_scale_factor;
  double           zoom_max_stepsize;
  double           zoom_area_factor;

  /* ROTATE */
  gboolean         mouseover_rotating;
  gboolean         exec_rotate_enable;
  gboolean         mouse_cursor_related_rotating;
  gboolean         rotate_on_object_adding;
  int              mouseover_rotate_velocity;
  int              exec_rotate_velocity;
  double           rotate_area_factor;
  double           rotate_scale_factor;

  /* BOUNCING */
  double           bounce_height;
  double           mouseover_bouncing_height;

  /* PRELIGHT */
  gboolean         draw_prelight;
  char            *prelight_color;
  double           prelight_alpha;
};

/* initiall default setting values */
void
kiba_settings_init_defaults (AkamaruModel *model,
                             KibaSettings *settings);

/* write config entrys from schema file to config file */
char*
kiba_update_settings_file (const char *kiba_home_dir);

/* get settings from settings_file */
void
kiba_get_settings (KibaDock   *dock,
                   const char *settings_file);

/* set the option setting_name from type setting_type in KibaSettings or AkamaruModel */
void
set_option (KibaDock   *dock,
            const char *setting_name,
            const char *setting_type,
            const char *setting_value);

/* set the option setting_name from type setting_type in option_struct */
void
kiba_plugin_set_option (KibaDock    *dock,
                        void        *option_struct,      /* plugin option struct */
                        const char  *setting_name,
                        const char  *setting_type,
                        const char  *setting_value,
                        GconfOption  options[],
                        int          array_elements);    /* options elements */

/* set all object related options in  option_struct */
void
kiba_object_set_options (KibaDock    *dock,
                         void        *option_struct,
                         GconfOption  gconf_options[],
                         int          array_elements);

/* turn autohide on/off */
void
toggle_auto_hide (GtkWidget *widget,
                  gpointer   data);  /* KibaDock* */

/* turn physics on/off */
void
toggle_physics (GtkWidget *widget,
                gpointer   data);  /* KibaDock* */

/* returns an allocated string from the config file */
char*
kiba_get_config_buffer (KibaDock *dock);

/* set setting_name in the settings file */
void
kiba_dock_option_changed (KibaDock   *dock,
                          const char *setting_name,
                          const char *setting_type,
                          const char *setting_value);

/* config changed callback */
void
kiba_config_changed_callback (KibaDock *dock);

#define define_struct_member(a,b)   struct { const char *name; size_t offset; } options[] = {\
    { "elasticity", G_STRUCT_OFFSET (b, elasticity) },\
    { "friction", G_STRUCT_OFFSET (b, friction) },\
    { "k", G_STRUCT_OFFSET (b, k) },\
    { "animated_background", G_STRUCT_OFFSET (a, animated_background) },\
    { "text_position_stepsize", G_STRUCT_OFFSET (a, text_position_stepsize) },\
    { "dock_position_stepsize", G_STRUCT_OFFSET (a, dock_position_stepsize) },\
    { "dock_position", G_STRUCT_OFFSET (a, dock_position) },\
    { "arrow_background", G_STRUCT_OFFSET (a, arrow_background) },\
    { "auto_hide_enable", G_STRUCT_OFFSET (a, auto_hide_enable) },\
    { "auto_hide_delay", G_STRUCT_OFFSET (a, auto_hide_delay) },\
    { "auto_hide_speed", G_STRUCT_OFFSET (a, auto_hide_speed) },\
    { "auto_hide_area", G_STRUCT_OFFSET (a, auto_hide_area) },\
    { "border_alpha", G_STRUCT_OFFSET (a, border_alpha) },\
    { "dock_background_alpha", G_STRUCT_OFFSET (a, dock_background_alpha) },\
    { "dock_border_color", G_STRUCT_OFFSET (a, dock_border_color) },\
    { "text_border_style", G_STRUCT_OFFSET (a, text_border_style) },\
    { "border_width", G_STRUCT_OFFSET (a, border_width) },\
    { "bounce_height", G_STRUCT_OFFSET (a, bounce_height) },\
    { "bubble_color", G_STRUCT_OFFSET (a, bubble_color) },\
    { "bubble_alpha", G_STRUCT_OFFSET (a, bubble_alpha) },\
    { "bubble_text_color", G_STRUCT_OFFSET (a, bubble_text_color) },\
    { "bubble_text_alpha", G_STRUCT_OFFSET (a, bubble_text_alpha) },\
    { "bubble_icon_edge", G_STRUCT_OFFSET (a, bubble_icon_edge) },\
    { "exec_rotate_velocity", G_STRUCT_OFFSET (a, exec_rotate_velocity) },\
    { "dock_cap_size", G_STRUCT_OFFSET (a, dock_cap_size) },\
    { "dock_cap_margin", G_STRUCT_OFFSET (a, dock_cap_margin) },\
    { "dock_alpha_1", G_STRUCT_OFFSET (a, dock_alpha_1) },\
    { "dock_alpha_2", G_STRUCT_OFFSET (a, dock_alpha_2) },\
    { "dock_alpha_3", G_STRUCT_OFFSET (a, dock_alpha_3) },\
    { "draw_dock", G_STRUCT_OFFSET (a, draw_dock) },\
    { "draw_prelight", G_STRUCT_OFFSET (a, draw_prelight) },\
    { "exec_rotate_enable", G_STRUCT_OFFSET (a, exec_rotate_enable) },\
    { "dock_bg_step_size", G_STRUCT_OFFSET (a, dock_bg_step_size) },\
    { "text_bg_step_size", G_STRUCT_OFFSET (a, text_bg_step_size) },\
    { "arrow_bg_step_size", G_STRUCT_OFFSET (a, arrow_bg_step_size) },\
    { "mouse_translation", G_STRUCT_OFFSET (a, mouse_translation) },\
    { "dock_background", G_STRUCT_OFFSET (a, dock_background) },\
    { "dock_bg_affection", G_STRUCT_OFFSET (a, dock_bg_affection) },\
    { "arrow_bg_affection", G_STRUCT_OFFSET (a, arrow_bg_affection) },\
    { "text_bg_affection", G_STRUCT_OFFSET (a, text_bg_affection) },\
    { "dock_bg_radius_1", G_STRUCT_OFFSET (a, dock_bg_radius_1) },\
    { "dock_bg_radius_2", G_STRUCT_OFFSET (a, dock_bg_radius_2) },\
    { "text_bg_radius_1", G_STRUCT_OFFSET (a, text_bg_radius_1) },\
    { "arrow_bg_radius_2", G_STRUCT_OFFSET (a, arrow_bg_radius_2) },\
    { "arrow_bg_radius_1", G_STRUCT_OFFSET (a, arrow_bg_radius_1) },\
    { "text_bg_radius_2", G_STRUCT_OFFSET (a, text_bg_radius_2) },\
    { "dock_color_1", G_STRUCT_OFFSET (a, dock_color_1) },\
    { "dock_color_2", G_STRUCT_OFFSET (a, dock_color_2) },\
    { "dock_color_3", G_STRUCT_OFFSET (a, dock_color_3) },\
    { "dock_corner_topleft", G_STRUCT_OFFSET (a, dock_corner_topleft) },\
    { "dock_corner_topright", G_STRUCT_OFFSET (a, dock_corner_topright) },\
    { "dock_corner_bottomleft", G_STRUCT_OFFSET (a, dock_corner_bottomleft) },\
    { "dock_corner_bottomright", G_STRUCT_OFFSET (a, dock_corner_bottomright) },\
    { "gravity_multiplier", G_STRUCT_OFFSET (a, gravity_multiplier) },\
    { "inertia", G_STRUCT_OFFSET (a, inertia) },\
    { "dock_margin", G_STRUCT_OFFSET (a, dock_margin) },\
    { "mouse_cursor_related_rotating", G_STRUCT_OFFSET (a, mouse_cursor_related_rotating) },\
    { "mouse_over_effect", G_STRUCT_OFFSET (a, mouse_over_effect) },\
    { "mouseover_rotating", G_STRUCT_OFFSET (a, mouseover_rotating) },\
    { "mouseover_rotate_velocity", G_STRUCT_OFFSET (a, mouseover_rotate_velocity) },\
    { "notification_rectangle_alpha", G_STRUCT_OFFSET (a, notification_rectangle_alpha) },\
    { "notification_rectangle_color", G_STRUCT_OFFSET (a, notification_rectangle_color) },\
    { "pulse_exec_enable", G_STRUCT_OFFSET (a, pulse_exec_enable) },\
    { "pulse_max_alpha", G_STRUCT_OFFSET (a, pulse_max_alpha) },\
    { "wobbly_object_spring", G_STRUCT_OFFSET (a, wobbly_object_spring) },\
    { "model", G_STRUCT_OFFSET (a, object_model) },\
    { "monitor_number", G_STRUCT_OFFSET (a, monitor_number) },\
    { "mouseover_bouncing_height", G_STRUCT_OFFSET (a, mouseover_bouncing_height) },\
    { "object_space", G_STRUCT_OFFSET (a, object_space) },\
    { "object_margin", G_STRUCT_OFFSET (a, object_margin) },\
    { "object_animate_stepsize", G_STRUCT_OFFSET (a, object_animate_stepsize) },\
    { "plugin_space", G_STRUCT_OFFSET (a, plugin_space) },\
    { "prelight_alpha", G_STRUCT_OFFSET (a, prelight_alpha) },\
    { "prelight_color", G_STRUCT_OFFSET (a, prelight_color) },\
    { "pulse_stepsize", G_STRUCT_OFFSET (a, pulse_stepsize) },\
    { "pulse_exec_stepsize", G_STRUCT_OFFSET (a, pulse_exec_stepsize) },\
    { "pulse_exec_zoom_factor", G_STRUCT_OFFSET (a, pulse_exec_zoom_factor) },\
    { "dock_edge_radius", G_STRUCT_OFFSET (a, dock_edge_radius) },\
    { "rope_object_distance", G_STRUCT_OFFSET (a, rope_object_distance) },\
    { "use_allways_physics", G_STRUCT_OFFSET (a, use_allways_physics) },\
    { "rope_gravity", G_STRUCT_OFFSET (a, rope_gravity) },\
    { "rope_spring_while_dragging", G_STRUCT_OFFSET (a, rope_spring_while_dragging) },\
    { "rotate_area_factor", G_STRUCT_OFFSET (a, rotate_area_factor) },\
    { "rotate_scale_factor", G_STRUCT_OFFSET (a, rotate_scale_factor) },\
    { "rotate_on_object_adding", G_STRUCT_OFFSET (a, rotate_on_object_adding) },\
    { "zoom_scale_factor", G_STRUCT_OFFSET (a, zoom_scale_factor) },\
    { "pulse_scale_factor", G_STRUCT_OFFSET (a, pulse_scale_factor) },\
    { "zoom_max_stepsize", G_STRUCT_OFFSET (a, zoom_max_stepsize) },\
    { "zoom_area_factor", G_STRUCT_OFFSET (a, zoom_area_factor) },\
    { "show_all_borders", G_STRUCT_OFFSET (a, show_all_borders) },\
    { "text_color", G_STRUCT_OFFSET (a, text_color) },\
    { "text_alpha", G_STRUCT_OFFSET (a, text_alpha) },\
    { "text_shadow_color", G_STRUCT_OFFSET (a, text_shadow_color) },\
    { "text_shadow_alpha", G_STRUCT_OFFSET (a, text_shadow_alpha) },\
    { "text_background", G_STRUCT_OFFSET (a, text_background) },\
    { "text_shadow_offset", G_STRUCT_OFFSET (a, text_shadow_offset) },\
    { "text_bg_color_1", G_STRUCT_OFFSET (a, text_bg_color_1) },\
    { "text_bg_color_2", G_STRUCT_OFFSET (a, text_bg_color_2) },\
    { "text_bg_color_3", G_STRUCT_OFFSET (a, text_bg_color_3) },\
    { "text_bg_color_4", G_STRUCT_OFFSET (a, text_bg_color_4) },\
    { "text_bg_color_5", G_STRUCT_OFFSET (a, text_bg_color_5) },\
    { "text_bg_alpha_1", G_STRUCT_OFFSET (a, text_bg_alpha_1) },\
    { "text_bg_alpha_2", G_STRUCT_OFFSET (a, text_bg_alpha_2) },\
    { "text_bg_alpha_3", G_STRUCT_OFFSET (a, text_bg_alpha_3) },\
    { "text_bg_alpha_4", G_STRUCT_OFFSET (a, text_bg_alpha_4) },\
    { "text_bg_alpha_5", G_STRUCT_OFFSET (a, text_bg_alpha_5) },\
    { "text_border_color", G_STRUCT_OFFSET (a, text_border_color) },\
    { "text_border_alpha", G_STRUCT_OFFSET (a, text_border_alpha) },\
    { "text_border_width", G_STRUCT_OFFSET (a, text_border_width) },\
    { "text_hide_delay", G_STRUCT_OFFSET (a, text_hide_delay) },\
    { "text_font_size", G_STRUCT_OFFSET (a, text_font_size) },\
    { "text_font_type", G_STRUCT_OFFSET (a, text_font_type) },\
    { "text_icon_padding", G_STRUCT_OFFSET (a, text_icon_padding) },\
    { "text_paint", G_STRUCT_OFFSET (a, text_paint) },\
    { "text_placement_x", G_STRUCT_OFFSET (a, text_placement_x) },\
    { "text_placement_y", G_STRUCT_OFFSET (a, text_placement_y) },\
    { "text_font_weight", G_STRUCT_OFFSET (a, text_font_weight) },\
    { "throw_gravity", G_STRUCT_OFFSET (a, throw_gravity) }\
  }

#endif
